package cc.mrbird.febs.auth.configure;

import cn.hutool.core.util.StrUtil;
import cn.hutool.crypto.SmUtil;
import cn.hutool.crypto.asymmetric.KeyType;
import cn.hutool.crypto.asymmetric.SM2;
import com.ruoyi.common.core.exception.BusinessException;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpHeaders;
import org.springframework.security.authentication.BadCredentialsException;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;
import org.springframework.web.filter.OncePerRequestFilter;

import javax.servlet.FilterChain;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;

import static com.ruoyi.common.core.constant.HttpStatus.UNAUTHORIZED;
import static com.ruoyi.common.core.constant.ThirdLoginConstants.IS_THIRD;
import static com.ruoyi.common.core.constant.ThirdLoginConstants.THIRD_FLAG;
import static com.ruoyi.common.core.utils.ServletUtils.renderString;
import static org.springframework.security.web.authentication.www.BasicAuthenticationConverter.AUTHENTICATION_SCHEME_BASIC;

@Slf4j
@Component
public class DecryptThirdTokenFilter extends OncePerRequestFilter {

    @Value("${crypt.privateKey}")
    private String privateKey;

    @Override
    protected void doFilterInternal(HttpServletRequest request, HttpServletResponse response, FilterChain chain) throws ServletException, IOException {
        // 非第三方获取token直接放行
        String isThird = request.getHeader(IS_THIRD);
        if(!THIRD_FLAG.equals(isThird)) {
            chain.doFilter(request, response);
            return;
        }
        // 第三方获取token,解密认证信息
        HeaderMapRequestWrapper requestWrapper;
        try {
            String header = request.getHeader(HttpHeaders.AUTHORIZATION);
            if (header == null) {
                throw new BadCredentialsException("No basic authentication token");
            } else {
                header = header.trim();
                if (!StringUtils.startsWithIgnoreCase(header, AUTHENTICATION_SCHEME_BASIC)) {
                    throw new BadCredentialsException("Need basic authentication token");
                }
                if (header.equalsIgnoreCase(AUTHENTICATION_SCHEME_BASIC)) {
                    throw new BadCredentialsException("Empty basic authentication token");
                }
                String token = header.substring(6);
                SM2 sm2 = SmUtil.sm2(privateKey, null);
                String decryptStr;
                try {
                    decryptStr = StrUtil.utf8Str(sm2.decryptFromBcd(token, KeyType.PrivateKey));
                } catch (Exception e) {
                    throw new BusinessException("解密失败");
                }
                requestWrapper = new HeaderMapRequestWrapper(request);
                requestWrapper.addHeader(HttpHeaders.AUTHORIZATION, AUTHENTICATION_SCHEME_BASIC + " " + decryptStr);
            }
        } catch (BadCredentialsException | BusinessException e) {
            log.error(e.getMessage());
            renderString(response, UNAUTHORIZED, e.getMessage());
            return;
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            response.setStatus(UNAUTHORIZED);
            return;
        }
        chain.doFilter(requestWrapper, response);
    }
}
