package com.ruoyi.common.core.enums;

/**
 * 接口返回状态代码的枚举类
 *
 * @author weixiaohuai
 */
public enum SignResultCodeEnum {
    /**
     * 操作成功 {@code code=200}
     */
    SUCCESS(200, "操作成功"),

    /**
     * 参数解析失败 {@code code=40000}
     */
    PARAMETER_ANALYSIS_FAILURE(40000, "参数解析失败"),

    /**
     * 接口授权认证失败，签名不匹配 {@code code=40001}
     */
    SIGNATURE_MISMATCH(40001, "接口授权认证失败，签名不匹配"),

    /**
     * 参数不足 {@code code=40002}
     */
    NULL_PARAMS_DATA(40002, "参数不足"),

    /**
     * 参数值为空 {@code code=40003}
     */
    NULL_PARAMS_VALUE(40003, "参数值为空"),

    /**
     * 缺少timestamp参数 {@code code=40004}
     */
    MISSING_TIMESTAMP(40004, "缺少timestamp参数"),

    /**
     * 缺少nonceStr参数 {@code code=40005}
     */
    MISSING_NONCESTR(40005, "缺少nonceStr参数"),

    /**
     * 缺少clientId参数 {@code code=40006}
     */
    MISSING_CLIENT_ID(40006, "缺少clientId参数"),

    /**
     * clientId错误 {@code code=40007}
     */
    MISSING_CLIENT_SECRET(40007, "clientId错误"),

    /**
     * 缺少signature参数 {@code code=40008}
     */
    MISSING_SIGNATURE(40008, "缺少signature参数"),
    /**
     * 时间戳已过期 {@code code=40009}
     */
    TIMESTAMP_OUT(40009, "时间戳已过期"),

    /**
     * 服务器执行错误 {@code code=500}
     */
    ERROR(500, "服务器执行错误"),

    /**
     * 请求数据不存在，或数据集为空 {@code code=60000}
     */
    NULL_RESULT_DATA(60000, "请求数据不存在，或数据集为空");

    private int value;
    private String text;

    private SignResultCodeEnum(int value, String text) {
        this.value = value;
        this.text = text;
    }

    /**
     * 获取value
     *
     * @return int
     */
    public int getValue() {
        return value;
    }

    /**
     * 设置 value
     *
     * @param value int
     */
    public void setValue(int value) {
        this.value = value;
    }

    /**
     * 获取text
     *
     * @return String
     */
    public String getText() {
        return text;
    }

    /**
     * 设置 text
     *
     * @param text String
     */
    public void setText(String text) {
        this.text = text;
    }
}
